<?php
namespace HellixElementor\Modules\Carousel\Widgets;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Image_Size;
use Elementor\Repeater;
use HellixElementor\Base\Base_Widget;
use Elementor\Plugin;


if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Carousel Base.
 */
abstract class Base extends Base_Widget {

	/**
	 * Number of Slides.
	 *
	 * @var int
	 */
	private $slide_prints_count = 0;

	/**
	 * Fetch the Scripts based on keyword.
	 *
	 * @return array
	 */
	public function get_script_depends() {
		return [ 'imagesloaded' ];
	}

	/**
	 * Add repeater controls for carousels.
	 *
	 * @param Repeater $repeater repeater arguments.
	 * @return array
	 */
	abstract protected function add_repeater_controls( Repeater $repeater );

	/**
	 * Default repeater values.
	 *
	 * @return array
	 */
	abstract protected function get_repeater_defaults();

	/**
	 * Print each item in the carousel
	 *
	 * @param array  $slide        Repeater setting for each slide.
	 * @param array  $settings     Widget settings.
	 * @param string $element_key  Key of the slide in the repeater.
	 */
	abstract protected function print_slide( array $slide, array $settings, $element_key );

	/**
	 * Register controls for this widget.
	 */
	protected function register_controls() {
		$this->register_content_tab_controls();
		$this->register_style_tab_controls();
	}

	/**
	 * Register controls that are loaded in the Content tab.
	 */
	protected function register_content_tab_controls() {
		$this->start_controls_section(
			'section_slides',
			[
				'label' => esc_html__( 'Slides', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
			]
		);

		$repeater = new Repeater();

		$this->add_repeater_controls( $repeater );

		$this->add_control(
			'slides',
			[
				'label'     => esc_html__( 'Slides', 'hellix-elementor' ),
				'type'      => Controls_Manager::REPEATER,
				'fields'    => $repeater->get_controls(),
				'default'   => $this->get_repeater_defaults(),
				'separator' => 'after',
			]
		);

		$this->add_control(
			'title_tag',
			[
				'label'   => esc_html__( 'Title HTML Tag', 'hellix-elementor' ),
				'type'    => Controls_Manager::SELECT,
				'options' => [
					'h1'   => 'H1',
					'h2'   => 'H2',
					'h3'   => 'H3',
					'h4'   => 'H4',
					'h5'   => 'H5',
					'h6'   => 'H6',
					'div'  => 'div',
					'span' => 'span',
					'p'    => 'p',
				],
				'default' => 'h3',
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'section_additional_options',
			[
				'label' => esc_html__( 'Additional Options', 'hellix-elementor' ),
			]
		);

		$slides_per_view = range( 1, 10 );
		$slides_per_view = array_combine( $slides_per_view, $slides_per_view );

		$this->add_responsive_control(
			'slides_per_view',
			[
				'type'      => Controls_Manager::SELECT,
				'label'     => esc_html__( 'Slides Per View', 'hellix-elementor' ),
				'options'   => $slides_per_view,
				'default'   => '1',
				'condition' => [
					'_skin!' => [ 'testimonial-style-1' ],
				],
			]
		);

		$this->add_control(
			'gutter',
			[
				'label'      => esc_html__( 'Gutter Padding', 'hellix-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'range'      => [
					'px' => [
						'min' => 10,
						'max' => 100,
					],
				],
				'size_units' => [ 'px' ],
				'default'    => [
					'size' => 30,
					'unit' => 'px',
				],
				'condition'  => [
					'_skin!' => [ 'testimonial-style-1' ],
				],
			]
		);

		$this->add_control(
			'show_arrows',
			[
				'type'      => Controls_Manager::SWITCHER,
				'label'     => esc_html__( 'Enable Arrows', 'hellix-elementor' ),
				'default'   => 'yes',
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'condition' => [
					'_skin!' => [ 'testimonial-style-2', 'hero-caorusel-10', 'hero-caorusel-1', 'hero-caorusel-7' ],
				],
			]
		);

		$this->add_control(
			'arrow_color',
			[
				'label'     => esc_html__( 'Arrow Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .sectionSlider .nav.-slider .nav__item .icon' => 'color: {{VALUE}}',
					'{{WRAPPER}} .sectionSlider .nav.-slider .nav__item' => 'border-color: {{VALUE}}',
				],
				'condition' => [
					'show_arrows' => 'yes',
					'_skin!'      => [ 'testimonial-style-2' ],
				],
			]
		);

		$this->add_control(
			'arrow_hover_color',
			[
				'label'     => esc_html__( 'Arrow Hover Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .sectionSlider .nav.-slider .nav__item:hover' => 'border-color: {{VALUE}}',
					'{{WRAPPER}} .sectionSlider .nav.-slider .nav__item:hover .icon' => 'color: {{VALUE}}',
				],
				'condition' => [
					'show_arrows' => 'yes',
					'_skin!'      => [ 'testimonial-style-2' ],
				],
			]
		);

		$this->add_control(
			'show_dots',
			[
				'type'      => Controls_Manager::SWITCHER,
				'label'     => esc_html__( 'Enable Dots', 'hellix-elementor' ),
				'default'   => 'yes',
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'condition' => [
					'_skin!' => [ 'hero-caorusel-4', 'hero-caorusel-5' ],
				],
			]
		);

		$this->add_control(
			'pagination_color',
			[
				'label'     => esc_html__( 'Dot Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .pagination.-slider .pagination__item' => 'background-color: {{VALUE}}',
					'{{WRAPPER}} .pagination.-slider .pagination__item.is-active::after' => 'border-color: {{VALUE}}',
				],
				'condition' => [
					'show_dots' => 'yes',
					'_skin!'    => [ 'hero-caorusel-1', 'hero-caorusel-3', 'hero-caorusel-4', 'hero-caorusel-2', 'hero-caorusel-5' ],
				],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Register controls that are loaded in the Style tab.
	 */
	protected function register_style_tab_controls() {

		$this->start_controls_section(
			'section_navigation',
			[
				'label' => esc_html__( 'Navigation', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Get Image HTML.
	 *
	 * @param array  $slide repeater single control arguments.
	 * @param string $index index.
	 * @return array
	 */
	public function get_slide_image_html( $slide, $index ) {

		$this->add_render_attribute(
			$index . '-image',
			[
				'data-parallax-target' => '',
				'class'                => 'swiper-lazy',
			]
		);

		$replace  = '<img ' . $this->get_render_attribute_string( $index . '-image' );
		$find     = '<img';
		$img_html = str_replace( $find, $replace, Group_Control_Image_Size::get_attachment_image_html( $slide, 'image_size', 'image' ) );

		return $img_html;
	}

	 /**
	  * Carousel Settings.
	  *
	  * @param array $settings settings.
	  * @return array
	  */
	public function get_slick_carousel_options( array $settings = null ) {

		$column      = ! empty( $settings['slides_per_view_mobile'] ) ? 'sm-' . intval( $settings['slides_per_view_mobile'] ) : 'sm-1';
		$column_md   = ! empty( $settings['slides_per_view_tablet'] ) ? 'md-' . intval( $settings['slides_per_view_tablet'] ) : 'md-2';
		$column_lg   = ! empty( $settings['slides_per_view'] ) ? 'lg-' . intval( $settings['slides_per_view'] ) : 'lg-4';
		$column_base = ! empty( $settings['slides_per_view'] ) ? 'base-' . intval( $settings['slides_per_view'] ) : 'base-4';

		$default_carousel_args = $column_base . ' ' . $column_lg . ' ' . $column_md . ' ' . $column;

		return $default_carousel_args;
	}

	/**
	 * Render script in the editor.
	 */
	public function render_script() {
		if ( Plugin::$instance->editor->is_edit_mode() ) :
			?>
			<script type="text/javascript">
				
				 /**
				 * Editor Slider Load.
				 */
				function SectionSlider() {
					var sectionSlider = document.querySelectorAll('.js-sectionSlider');
					if (!sectionSlider.length) return;

					for (let i = 0; i < sectionSlider.length; i++) {
						const el = sectionSlider[i];
						
						let gap = 0;
						let loop = false;
						let centered = false;
						let pagination = false;

						if (el.getAttribute('data-gap'))    gap = el.getAttribute('data-gap');
						if (el.hasAttribute('data-loop'))   loop = true;
						if (el.hasAttribute('data-center')) centered = true;

						if (el.hasAttribute('data-pagination')) {
						pagination = {
							el: el.querySelector('.js-pagination'),
							bulletClass: 'pagination__item',
							bulletActiveClass: 'is-active',
							bulletElement: 'div',
							clickable: true
						};
						}
					
						const colsArray = el.getAttribute('data-slider-col').split(' ');

						let cols_base = 1;
						let cols_lg = 1;
						let cols_md = 1;
						let cols_sm = 1;

						colsArray.forEach(el => {
						if (el.includes('base')) cols_base = el.slice(-1);
						if (el.includes('lg')) cols_lg = el.slice(-1);
						if (el.includes('md')) cols_md = el.slice(-1);
						if (el.includes('sm')) cols_sm = el.slice(-1);
						});

						new Swiper(el, {
						speed: 800,
						autoHeight: true,
						spaceBetween: parseInt(gap),
						centeredSlides: centered,
						parallax: true,
						watchSlidesVisibility: true,

						loop: loop,
						loopAdditionalSlides: 1,
						preloadImages: false,
						lazy: true,
						
						lazy: {
							loadPrevNext: true,
						},

						slidesPerView: parseInt(cols_base),

						breakpoints: {
							1199: { slidesPerView: parseInt(cols_lg) },
							991:  { slidesPerView: parseInt(cols_md) },
							767:  { slidesPerView: parseInt(cols_sm) },
						},

						navigation: {
							prevEl: el.querySelector('.js-prev'),
							nextEl: el.querySelector('.js-next'),
						},

						pagination: pagination,
						});
					}
					}

					// For Lazy Loading Images.
					function lazyLoading() {
					if (!document.querySelector('.js-lazy')) {
						return;
					}

					new LazyLoad({
						elements_selector: ".js-lazy",
					});
					}
					
					// Initiate js functions.
					SectionSlider();
					lazyLoading();
			</script>
			<?php
		endif;
	}

}
