<?php
namespace HellixElementor\Modules\IconBox\Skins;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use Elementor;
use Elementor\Icons_Manager;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Skin_Base;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Background;
use HellixElementor\Plugin;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use HellixElementor\Core\Utils as HXUtils;

/**
 * Skin Icon Box - Card
 */
class Skin_Icon_Box_Style_1 extends Skin_Base {

	/**
	 * Get the id of the skin.
	 *
	 * @return string
	 */
	public function get_id() {
		return 'card-hellix';
	}

	/**
	 * Get the title of the skin.
	 *
	 * @return string
	 */
	public function get_title() {
		return esc_html__( 'Card - Hellix', 'hellix-elementor' );
	}

	/**
	 * Constructor.
	 *
	 * @param Elementor\Widget_Base $parent The widget settings.
	 * @return void
	 */
	public function __construct( Elementor\Widget_Base $parent ) {
		parent::__construct( $parent );
		add_filter( 'elementor/widget/print_template', array( $this, 'skin_print_template' ), 10, 2 );
		add_action( 'elementor/element/icon-box/section_icon/before_section_end', [ $this, 'register_content_icon_controls' ], 10 );

	}

	/**
	 * Register Content Icon controls.
	 *
	 * @param Elementor\Widget_Base $widget The widget settings.
	 */
	public function register_content_icon_controls( $widget ) {

		$update_control_ids = [ 'view', 'position' ];

		foreach ( $update_control_ids as $update_control_id ) {
			$widget->update_control(
				$update_control_id,
				[
					'condition' => [
						'_skin!' => [ 'card-hellix', 'hellix-style-2', 'hellix-style-3' ],
					],
				]
			);
		}

		$widget->add_control(
			'btn_txt',
			[
				'label'       => esc_html__( 'Button text', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'description' => esc_html__( 'Applied to button text', 'hellix-elementor' ),
				'condition'   => [ '_skin' => [ 'card-hellix', 'hellix-style-3' ] ],

			]
		);

		$widget->add_control(
			'bg_number',
			[
				'label'       => esc_html__( 'Background Number', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'description' => esc_html__( 'Enter number eg:01', 'hellix-elementor' ),
				'condition'   => [ '_skin' => 'hellix-style-2' ],
			]
		);

	}

	/**
	 * Render the skin output.
	 *
	 * @return void
	 */
	public function render() {
		$widget       = $this->parent;
		$settings     = $widget->get_settings_for_display();
		$icon_wrapper = $settings['icon_wraper_css'];

		if ( ! isset( $settings['icon'] ) ) {
			$settings['icon'] = '';
		}

		$has_icon = ! empty( $settings['icon'] );

		if ( ! $has_icon && ! empty( $settings['selected_icon']['value'] ) ) {
			$has_icon = true;
		}
		$icon_wrapper = ! empty( $icon_wrapper ) ? 'service__icon ' . $icon_wrapper : 'service__icon';
		$widget->add_render_attribute( 'icon-wrapper-css', 'class', $icon_wrapper );
		?> 
		<div class="service -card -hover">
			<div class="service__content -border">

				<?php if ( $has_icon ) : ?>
					<div <?php $widget->print_render_attribute_string( 'icon-wrapper-css' ); ?>>
						<?php $this->render_icon( $settings ); ?>
					</div>
				<?php endif; ?>

				<?php
				if ( ! empty( $settings['title_text'] ) ) {
					$this->render_title( $settings );
				}

				if ( ! empty( $settings['description_text'] ) ) {
					$this->render_description( $settings );
				}

				if ( ! empty( $settings['btn_txt'] ) ) {
					$this->render_button( $settings );
				}
				?>
			</div>
		</div>
		<?php
	}


	/**
	 * Skin Print Template.
	 *
	 * @param string                $content The skin output string.
	 * @param Elementor\Widget_Base $widget The widget.
	 *
	 * @return string
	 */
	public function skin_print_template( $content, $widget ) {
		if ( 'icon-box' === $widget->get_name() ) {
			return '';
		}
		return $content;
	}

	/**
	 * Render Icon.
	 *
	 * @param array $settings The widget settings.
	 */
	public function render_icon( $settings ) {
		$widget   = $this->parent;
		$icon_css = $settings['icon_css'];

		Icons_Manager::render_icon(
			$settings['selected_icon'],
			[
				'aria-hidden' => 'true',
				'class'       => [ 'elementor-icon elementor-icon-box icon ', $icon_css ],
			]
		);
	}

	/**
	 * Render button.
	 *
	 * @param array $settings The widget settings.
	 */
	public function render_button( $settings ) {
		$widget  = $this->parent;
		$btn_txt = $settings['btn_txt'];

		$btn_size = '';
		if ( 'simple' !== $settings['button_type'] ) {
			$btn_size = '-md';
		}
		$class_btn = 'button__text_color button ' . $btn_size . ' -' . $settings['button_type'] . ' ';

		$widget->add_render_attribute( 'button', 'class', $class_btn );
		if ( ! empty( $settings['link']['url'] ) ) {
			$widget->add_link_attributes( 'button', $settings['link'] );
		}

		?>
		<div class="service__button">
			<a <?php $widget->print_render_attribute_string( 'button' ); ?>><?php echo esc_html( $btn_txt ); ?></a>
		</div> 
		<?php
	}

	/**
	 * Render title.
	 *
	 * @param array $settings The widget settings.
	 */
	public function render_title( $settings ) {

		$widget    = $this->parent;
		$title_css = $settings['title_css'];
		$tag       = $settings['title_size'];
		$widget->add_render_attribute( 'title', 'class', [ 'service__title elementor-icon-box-title' ] );

		if ( ! empty( $settings['title_css'] ) ) {
			$widget->add_render_attribute( 'title', 'class', $title_css );
		}
		?>
		<<?php echo esc_html( $tag ); ?> <?php $widget->print_render_attribute_string( 'title' ); ?>>
			<?php echo esc_html( $settings['title_text'] ); ?>
		</<?php echo esc_html( $tag ); ?> > 
		<?php
	}

	/**
	 * Render description.
	 *
	 * @param array $settings The widget settings.
	 */
	public function render_description( $settings ) {

		$widget   = $this->parent;
		$desc_css = $settings['description_css'];

		$widget->add_render_attribute( 'description_text', 'class', [ 'elementor-icon-box-description service__text' ] );

		if ( ! empty( $desc_css ) ) {
			$widget->add_render_attribute( 'description_text', 'class', $desc_css );
		}
		?>
		<p <?php $widget->print_render_attribute_string( 'description_text' ); ?>>
			<?php echo wp_kses_post( $settings['description_text'] ); ?>
		</p> 
		<?php
	}
}
