<?php
namespace HellixElementor\Modules\Posts\Skins;

use Elementor\Controls_Manager;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Skin_Base as Elementor_Skin_Base;
use Elementor\Widget_Base;
use HellixElementor\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * The posts skin base
 */
abstract class Skin_Base extends Elementor_Skin_Base {

	/**
	 * Current Permalink.
	 *
	 * @var string
	 */
	protected $current_permalink;

	/**
	 * Register control actions
	 */
	protected function _register_controls_actions() {
		add_action( 'elementor/element/hx-posts/section_layout/before_section_end', [ $this, 'register_controls' ] );
		add_action( 'elementor/element/hx-posts/section_query/after_section_end', [ $this, 'register_style_sections' ] );
	}

	/**
	 * Get WP Nav Lnk
	 *
	 * @param  Widget_Base $widget widget.
	 * @return void
	 */
	public function register_style_sections( Widget_Base $widget ) {
		$this->parent = $widget;

		$this->register_design_controls();
	}

	/**
	 * Register controls for the skin.
	 *
	 * @param Widget_Base $widget The widget instance.
	 */
	public function register_controls( Widget_Base $widget ) {
		$this->parent = $widget;

		$this->register_columns_controls();
		$this->register_post_count_control();
		$this->register_thumbnail_controls();
		$this->register_title_controls();
		$this->register_excerpt_controls();
		$this->register_meta_data_controls();
		$this->register_read_more_controls();
		$this->register_link_controls();
	}

	/**
	 * Register design control actions
	 */
	public function register_design_controls() {
		$this->register_design_layout_controls();
		$this->register_design_image_controls();
		$this->register_design_content_controls();
	}

	/**
	 * Register thumbnail controls
	 */
	protected function register_thumbnail_controls() {
		$this->add_control(
			'thumbnail',
			[
				'label'        => esc_html__( 'Image Position', 'hellix-elementor' ),
				'type'         => Controls_Manager::SELECT,
				'default'      => 'top',
				'options'      => [
					'top'   => esc_html__( 'Top', 'hellix-elementor' ),
					'left'  => esc_html__( 'Left', 'hellix-elementor' ),
					'right' => esc_html__( 'Right', 'hellix-elementor' ),
					'none'  => esc_html__( 'None', 'hellix-elementor' ),
				],
				'prefix_class' => 'elementor-posts--thumbnail-',
			]
		);

		$this->add_control(
			'masonry',
			[
				'label'              => esc_html__( 'Masonry', 'hellix-elementor' ),
				'type'               => Controls_Manager::SWITCHER,
				'label_off'          => esc_html__( 'Off', 'hellix-elementor' ),
				'label_on'           => esc_html__( 'On', 'hellix-elementor' ),
				'condition'          => [
					$this->get_control_id( 'columns!' )  => '1',
					$this->get_control_id( 'thumbnail' ) => 'top',
				],
				'render_type'        => 'ui',
				'frontend_available' => true,
			]
		);

		$this->add_group_control(
			Group_Control_Image_Size::get_type(),
			[
				'name'         => 'thumbnail_size',
				'default'      => 'medium',
				// 'exclude' => [ 'custom' ],
				'condition'    => [
					$this->get_control_id( 'thumbnail!' ) => 'none',
				],
				'prefix_class' => 'elementor-posts--thumbnail-size-',
			]
		);

		$this->add_responsive_control(
			'item_ratio',
			[
				'label'          => esc_html__( 'Image Ratio', 'hellix-elementor' ),
				'type'           => Controls_Manager::SLIDER,
				'default'        => [
					'size' => 0.66,
				],
				'tablet_default' => [
					'size' => '',
				],
				'mobile_default' => [
					'size' => 0.5,
				],
				'range'          => [
					'px' => [
						'min'  => 0.1,
						'max'  => 2,
						'step' => 0.01,
					],
				],
				'selectors'      => [
					'{{WRAPPER}} .elementor-posts-container .elementor-post__thumbnail' => 'padding-bottom: calc( {{SIZE}} * 100% );',
					'{{WRAPPER}}:after' => 'content: "{{SIZE}}";',
				],
				'condition'      => [
					$this->get_control_id( 'thumbnail!' ) => 'none',
					$this->get_control_id( 'masonry' )    => '',
				],
			]
		);

		$this->add_responsive_control(
			'image_width',
			[
				'label'          => esc_html__( 'Image Width', 'hellix-elementor' ),
				'type'           => Controls_Manager::SLIDER,
				'range'          => [
					'%'  => [
						'min' => 10,
						'max' => 100,
					],
					'px' => [
						'min' => 10,
						'max' => 600,
					],
				],
				'default'        => [
					'size' => 100,
					'unit' => '%',
				],
				'tablet_default' => [
					'size' => '',
					'unit' => '%',
				],
				'mobile_default' => [
					'size' => 100,
					'unit' => '%',
				],
				'size_units'     => [ '%', 'px' ],
				'selectors'      => [
					'{{WRAPPER}} .elementor-post__thumbnail__link' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition'      => [
					$this->get_control_id( 'thumbnail!' ) => 'none',
				],
			]
		);
	}

	/**
	 * Register columns controls
	 */
	protected function register_columns_controls() {
		$this->add_responsive_control(
			'columns',
			[
				'label'              => esc_html__( 'Columns', 'hellix-elementor' ),
				'type'               => Controls_Manager::SELECT,
				'default'            => '3',
				'tablet_default'     => '2',
				'mobile_default'     => '1',
				'options'            => [
					'1' => '1',
					'2' => '2',
					'3' => '3',
					'4' => '4',
					'5' => '5',
					'6' => '6',
				],
				'prefix_class'       => 'elementor-grid%s-',
				'frontend_available' => true,
			]
		);
	}

	/**
	 * Register post count controls.
	 */
	protected function register_post_count_control() {
		$this->add_control(
			'posts_per_page',
			[
				'label'   => esc_html__( 'Posts Per Page', 'hellix-elementor' ),
				'type'    => Controls_Manager::NUMBER,
				'default' => 6,
			]
		);
	}

	/**
	 * Register title controls
	 */
	protected function register_title_controls() {
		$this->add_control(
			'show_title',
			[
				'label'     => esc_html__( 'Title', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'default'   => 'yes',
				'separator' => 'before',
			]
		);

		$this->add_control(
			'title_tag',
			[
				'label'     => esc_html__( 'Title HTML Tag', 'hellix-elementor' ),
				'type'      => Controls_Manager::SELECT,
				'options'   => [
					'h1'   => 'H1',
					'h2'   => 'H2',
					'h3'   => 'H3',
					'h4'   => 'H4',
					'h5'   => 'H5',
					'h6'   => 'H6',
					'div'  => 'div',
					'span' => 'span',
					'p'    => 'p',
				],
				'default'   => 'h3',
				'condition' => [
					$this->get_control_id( 'show_title' ) => 'yes',
				],
			]
		);

	}

	/**
	 * Register excerpt controls
	 */
	protected function register_excerpt_controls() {
		$this->add_control(
			'show_excerpt',
			[
				'label'     => esc_html__( 'Excerpt', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'default'   => 'yes',
			]
		);

		$this->add_control(
			'excerpt_length',
			[
				'label'     => esc_html__( 'Excerpt Length', 'hellix-elementor' ),
				'type'      => Controls_Manager::NUMBER,
				/** This filter is documented in wp-includes/formatting.php */
				'default'   => apply_filters( 'excerpt_length', 25 ),
				'condition' => [
					$this->get_control_id( 'show_excerpt' ) => 'yes',
				],
			]
		);
	}

	/**
	 * Register Read more controls.
	 */
	protected function register_read_more_controls() {
		$this->add_control(
			'show_read_more',
			[
				'label'     => esc_html__( 'Read More', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'default'   => 'yes',
				'separator' => 'before',
			]
		);

		$this->add_control(
			'light_version',
			[
				'label'     => esc_html__( 'Light Version', 'hellix-elementor' ),
				'type'      => Controls_Manager::SWITCHER,
				'label_on'  => esc_html__( 'Show', 'hellix-elementor' ),
				'label_off' => esc_html__( 'Hide', 'hellix-elementor' ),
				'default'   => '',
				'condition' => [
					'_skin' => [ 'hellix-posts-v1', 'hellix-posts-v2' ],
				],
			]
		);

		$this->add_control(
			'read_more_text',
			[
				'label'     => esc_html__( 'Read More Text', 'hellix-elementor' ),
				'type'      => Controls_Manager::TEXT,
				'default'   => esc_html__( 'Read More', 'hellix-elementor' ),
				'condition' => [
					$this->get_control_id( 'show_read_more' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'date_css',
			[
				'label'   => esc_html__( 'Date CSS', 'hellix-elementor' ),
				'type'    => Controls_Manager::TEXT,
				'default' => 'uppercase',
			]
		);

		$this->add_control(
			'author_css',
			[
				'label'   => esc_html__( 'Author CSS', 'hellix-elementor' ),
				'type'    => Controls_Manager::TEXT,
				'default' => 'uppercase',
			]
		);
	}

	/**
	 * Register Link controls
	 */
	protected function register_link_controls() {
		$this->add_control(
			'open_new_tab',
			[
				'label'       => esc_html__( 'Open in new window', 'hellix-elementor' ),
				'type'        => Controls_Manager::SWITCHER,
				'label_on'    => esc_html__( 'Yes', 'hellix-elementor' ),
				'label_off'   => esc_html__( 'No', 'hellix-elementor' ),
				'default'     => 'no',
				'render_type' => 'none',
			]
		);
	}

	/**
	 * Get optional link attributes.
	 */
	protected function get_optional_link_attributes_html() {
		$settings                 = $this->parent->get_settings();
		$new_tab_setting_key      = $this->get_control_id( 'open_new_tab' );
		$optional_attributes_html = 'yes' === $settings[ $new_tab_setting_key ] ? 'target="_blank"' : '';

		return $optional_attributes_html;
	}

	/**
	 * Register meta data controls.
	 */
	protected function register_meta_data_controls() {
		$this->add_control(
			'meta_data',
			[
				'label'       => esc_html__( 'Meta Data', 'hellix-elementor' ),
				'label_block' => true,
				'type'        => Controls_Manager::SELECT2,
				'default'     => [ 'date', 'comments' ],
				'multiple'    => true,
				'options'     => [
					'author' => esc_html__( 'Author', 'hellix-elementor' ),
					'date'   => esc_html__( 'Date', 'hellix-elementor' ),
				],
				'separator'   => 'before',
			]
		);

		$this->add_control(
			'meta_separator',
			[
				'label'     => esc_html__( 'Separator Between', 'hellix-elementor' ),
				'type'      => Controls_Manager::TEXT,
				'default'   => '///',
				'selectors' => [
					'{{WRAPPER}} .elementor-post__meta-data span + span:before' => 'content: "{{VALUE}}"',
				],
				'condition' => [
					$this->get_control_id( 'meta_data!' ) => [],
				],
			]
		);
	}

	/**
	 * Style Tab
	 */
	protected function register_design_layout_controls() {
		$this->start_controls_section(
			'section_design_layout',
			[
				'label' => esc_html__( 'Layout', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'column_gap',
			[
				'label'     => esc_html__( 'Columns Gap', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'default'   => [
					'size' => 30,
				],
				'range'     => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}}' => '--grid-column-gap: {{SIZE}}{{UNIT}}',
				],
			]
		);

		$this->add_control(
			'row_gap',
			[
				'label'              => esc_html__( 'Rows Gap', 'hellix-elementor' ),
				'type'               => Controls_Manager::SLIDER,
				'default'            => [
					'size' => 35,
				],
				'range'              => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'frontend_available' => true,
				'selectors'          => [
					'{{WRAPPER}}' => '--grid-row-gap: {{SIZE}}{{UNIT}}',
				],
			]
		);

		$this->add_control(
			'alignment',
			[
				'label'        => esc_html__( 'Alignment', 'hellix-elementor' ),
				'type'         => Controls_Manager::CHOOSE,
				'options'      => [
					'left'   => [
						'title' => esc_html__( 'Left', 'hellix-elementor' ),
						'icon'  => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'hellix-elementor' ),
						'icon'  => 'eicon-text-align-center',
					],
					'right'  => [
						'title' => esc_html__( 'Right', 'hellix-elementor' ),
						'icon'  => 'eicon-text-align-right',
					],
				],
				'prefix_class' => 'elementor-posts--align-',
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Register design image controls.
	 */
	protected function register_design_image_controls() {
		$this->start_controls_section(
			'section_design_image',
			[
				'label'     => esc_html__( 'Image', 'hellix-elementor' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					$this->get_control_id( 'thumbnail!' ) => 'none',
				],
			]
		);

		$this->add_control(
			'img_border_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'hellix-elementor' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .elementor-post__thumbnail' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition'  => [
					$this->get_control_id( 'thumbnail!' ) => 'none',
				],
			]
		);

		$this->add_control(
			'image_spacing',
			[
				'label'     => esc_html__( 'Spacing', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}}.elementor-posts--thumbnail-left .elementor-post__thumbnail__link' => 'margin-right: {{SIZE}}{{UNIT}}',
					'{{WRAPPER}}.elementor-posts--thumbnail-right .elementor-post__thumbnail__link' => 'margin-left: {{SIZE}}{{UNIT}}',
					'{{WRAPPER}}.elementor-posts--thumbnail-top .elementor-post__thumbnail__link' => 'margin-bottom: {{SIZE}}{{UNIT}}',
				],
				'default'   => [
					'size' => 20,
				],
				'condition' => [
					$this->get_control_id( 'thumbnail!' ) => 'none',
				],
			]
		);

		$this->start_controls_tabs( 'thumbnail_effects_tabs' );

		$this->start_controls_tab(
			'normal',
			[
				'label' => esc_html__( 'Normal', 'hellix-elementor' ),
			]
		);

		$this->add_group_control(
			Group_Control_Css_Filter::get_type(),
			[
				'name'     => 'thumbnail_filters',
				'selector' => '{{WRAPPER}} .elementor-post__thumbnail img',
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'hover',
			[
				'label' => esc_html__( 'Hover', 'hellix-elementor' ),
			]
		);

		$this->add_group_control(
			Group_Control_Css_Filter::get_type(),
			[
				'name'     => 'thumbnail_hover_filters',
				'selector' => '{{WRAPPER}} .elementor-post:hover .elementor-post__thumbnail img',
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();
	}

	/**
	 * Register design content controls.
	 */
	protected function register_design_content_controls() {
		$this->start_controls_section(
			'section_design_content',
			[
				'label' => esc_html__( 'Content', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'heading_title_style',
			[
				'label'     => esc_html__( 'Title', 'hellix-elementor' ),
				'type'      => Controls_Manager::HEADING,
				'condition' => [
					$this->get_control_id( 'show_title' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'title_color',
			[
				'label'     => esc_html__( 'Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'global'    => [
					'default' => Global_Colors::COLOR_SECONDARY,
				],
				'selectors' => [
					'{{WRAPPER}} .hellix-post__title, {{WRAPPER}} .hellix-post__title a' => 'color: {{VALUE}};',
				],
				'condition' => [
					$this->get_control_id( 'show_title' ) => 'yes',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'      => 'title_typography',
				'global'    => [
					'default' => Global_Typography::TYPOGRAPHY_PRIMARY,
				],
				'selector'  => '{{WRAPPER}} .hellix-post__title, {{WRAPPER}} .hellix-post__title a',
				'condition' => [
					$this->get_control_id( 'show_title' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'title_spacing',
			[
				'label'     => esc_html__( 'Spacing', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .hellix-post__title' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					$this->get_control_id( 'show_title' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'heading_meta_style',
			[
				'label'     => esc_html__( 'Meta', 'hellix-elementor' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					$this->get_control_id( 'meta_data!' ) => [],
				],
			]
		);

		$this->add_control(
			'meta_color',
			[
				'label'     => esc_html__( 'Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .hellix-post__meta-data' => 'color: {{VALUE}};',
				],
				'condition' => [
					$this->get_control_id( 'meta_data!' ) => [],
				],
			]
		);

		$this->add_control(
			'meta_separator_color',
			[
				'label'     => esc_html__( 'Separator Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .hellix-post__meta-data span:before' => 'color: {{VALUE}};',
				],
				'condition' => [
					$this->get_control_id( 'meta_data!' ) => [],
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'      => 'meta_typography',
				'global'    => [
					'default' => Global_Typography::TYPOGRAPHY_SECONDARY,
				],
				'selector'  => '{{WRAPPER}} .hellix-post__meta-data',
				'condition' => [
					$this->get_control_id( 'meta_data!' ) => [],
				],
			]
		);

		$this->add_control(
			'meta_spacing',
			[
				'label'     => esc_html__( 'Spacing', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .hellix-post__meta-data' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					$this->get_control_id( 'meta_data!' ) => [],
				],
			]
		);

		$this->add_control(
			'heading_excerpt_style',
			[
				'label'     => esc_html__( 'Excerpt', 'hellix-elementor' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					$this->get_control_id( 'show_excerpt' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'excerpt_color',
			[
				'label'     => esc_html__( 'Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .elementor-post__excerpt p' => 'color: {{VALUE}};',
				],
				'condition' => [
					$this->get_control_id( 'show_excerpt' ) => 'yes',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'      => 'excerpt_typography',
				'global'    => [
					'default' => Global_Typography::TYPOGRAPHY_TEXT,
				],
				'selector'  => '{{WRAPPER}} .elementor-post__excerpt p',
				'condition' => [
					$this->get_control_id( 'show_excerpt' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'excerpt_spacing',
			[
				'label'     => esc_html__( 'Spacing', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .elementor-post__excerpt' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					$this->get_control_id( 'show_excerpt' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'heading_readmore_style',
			[
				'label'     => esc_html__( 'Read More', 'hellix-elementor' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
				'condition' => [
					$this->get_control_id( 'show_read_more' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'read_more_color',
			[
				'label'     => esc_html__( 'Color', 'hellix-elementor' ),
				'type'      => Controls_Manager::COLOR,
				'global'    => [
					'default' => Global_Colors::COLOR_ACCENT,
				],
				'selectors' => [
					'{{WRAPPER}} .hellix-post__read-more' => 'color: {{VALUE}};',
				],
				'condition' => [
					$this->get_control_id( 'show_read_more' ) => 'yes',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'      => 'read_more_typography',
				'selector'  => '{{WRAPPER}} .hellix-post__read-more',
				'global'    => [
					'default' => Global_Typography::TYPOGRAPHY_ACCENT,
				],
				'condition' => [
					$this->get_control_id( 'show_read_more' ) => 'yes',
				],
			]
		);

		$this->add_control(
			'read_more_spacing',
			[
				'label'     => esc_html__( 'Spacing', 'hellix-elementor' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'max' => 100,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .hellix-post__text' => 'margin-bottom: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
					$this->get_control_id( 'show_read_more' ) => 'yes',
				],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render.
	 */
	public function render() {
		$this->parent->query_posts();
		$query = $this->parent->get_query();

		if ( ! $query->found_posts ) {
			return;
		}

		$this->render_loop_header();

		// It's the global `wp_query` it self. and the loop was started from the theme.
		if ( $query->in_the_loop ) {
			$this->current_permalink = get_permalink();
			$this->render_post();
		} else {
			while ( $query->have_posts() ) {
				$query->the_post();

				$this->current_permalink = get_permalink();
				$this->render_post();
			}
		}

		wp_reset_postdata();
		$this->render_loop_footer();
		$this->render_script();

	}

	/**
	 * Filter Excerpt Length.
	 */
	public function filter_excerpt_length() {
		return $this->get_instance_value( 'excerpt_length' );
	}

	/**
	 * Filter Excerpt More.
	 *
	 * @param string $more more text.
	 */
	public function filter_excerpt_more( $more ) {
		return '';
	}

	/**
	 * Get Container Class.
	 */
	public function get_container_class() {
		return 'elementor-posts--skin-' . $this->get_id();
	}

	/**
	 * Render Thumbnail.
	 */
	protected function render_thumbnail() {
		$thumbnail = $this->get_instance_value( 'thumbnail' );

		if ( 'none' === $thumbnail && ! Plugin::elementor()->editor->is_edit_mode() ) {
			return;
		}

		$settings                 = $this->parent->get_settings();
		$setting_key              = $this->get_control_id( 'thumbnail_size' );
		$settings[ $setting_key ] = [
			'id' => get_post_thumbnail_id(),
		];
		$thumbnail_html           = Group_Control_Image_Size::get_attachment_image_html( $settings, $setting_key );

		if ( empty( $thumbnail_html ) ) {
			return;
		}

		$optional_attributes_html = $this->get_optional_link_attributes_html();

		?>
		<a class="elementor-post__thumbnail__link" href="<?php echo esc_url( $this->current_permalink ); ?>" <?php echo esc_attr( $optional_attributes_html ); ?>>
			<div class="elementor-post__thumbnail"><?php echo wp_kses_post( $thumbnail_html ); ?></div>
		</a>
		<?php
	}

	/**
	 * Render Title.
	 */
	protected function render_title() {
		if ( ! $this->get_instance_value( 'show_title' ) ) {
			return;
		}

		$optional_attributes_html = $this->get_optional_link_attributes_html();

		$tag = $this->get_instance_value( 'title_tag' );
		?>
		<<?php echo esc_html( $tag ); ?> class="elementor-post__title">
			<a href="<?php echo esc_url( $this->current_permalink ); ?>" <?php echo esc_attr( $optional_attributes_html ); ?>>
				<?php the_title(); ?>
			</a>
		</<?php echo esc_html( $tag ); ?>>
		<?php
	}

	/**
	 * Render Excerpt.
	 */
	protected function render_excerpt() {
		add_filter( 'excerpt_more', [ $this, 'filter_excerpt_more' ], 20 );
		add_filter( 'excerpt_length', [ $this, 'filter_excerpt_length' ], 20 );

		if ( ! $this->get_instance_value( 'show_excerpt' ) ) {
			return;
		}

		add_filter( 'excerpt_more', [ $this, 'filter_excerpt_more' ], 20 );
		add_filter( 'excerpt_length', [ $this, 'filter_excerpt_length' ], 20 );

		?>
		<div class="elementor-post__excerpt">
			<?php the_excerpt(); ?>
		</div>
		<?php

		remove_filter( 'excerpt_length', [ $this, 'filter_excerpt_length' ], 20 );
		remove_filter( 'excerpt_more', [ $this, 'filter_excerpt_more' ], 20 );
	}

	/**
	 * Render Read More.
	 */
	protected function render_read_more() {
		if ( ! $this->get_instance_value( 'show_read_more' ) ) {
			return;
		}

		$optional_attributes_html = $this->get_optional_link_attributes_html();

		?>
			<a class="elementor-post__read-more" href="<?php echo esc_url( $this->current_permalink ); ?>" <?php echo esc_attr( $optional_attributes_html ); ?>>
				<?php echo wp_kses_post( $this->get_instance_value( 'read_more_text' ) ); ?>
			</a>
		<?php
	}

	/**
	 * Render Post Header.
	 */
	protected function render_post_header() {
		?>
		<article <?php post_class( [ 'elementor-post elementor-grid-item' ] ); ?>>
		<?php
	}

	/**
	 * Render Post Footer.
	 */
	protected function render_post_footer() {
		?>
		</article>
		<?php
	}

	/**
	 * Render Text Header.
	 */
	protected function render_text_header() {
		?>
		<div class="elementor-post__text">
		<?php
	}

	/**
	 * Render Text Footer.
	 */
	protected function render_text_footer() {
		?>
		</div>
		<?php
	}

	/**
	 * Render Loop Header.
	 */
	protected function render_loop_header() {

		$this->parent->add_render_attribute(
			'container',
			[
				'class' => 'masonry -gap-32 -col-3 layout-pt-sm js-masonry js-masonry-no-filter',
			]
		);

		?>
		<div <?php $this->parent->print_render_attribute_string( 'container' ); ?>>
		<?php
	}

	/**
	 * Render Loop Footer.
	 */
	protected function render_loop_footer() {
		?>
		</div>
		<?php
		$this->parent->hellix_pagination();
	}

	/**
	 * Render Meta Data.
	 */
	protected function render_meta_data() {
		$settings = $this->get_instance_value( 'meta_data' );

		if ( empty( $settings ) ) {
			return;
		}

		if ( in_array( 'date', $settings ) ) {
			$this->render_date_by_type();
		}

		if ( in_array( 'author', $settings ) ) {
			$this->render_author();
		}
	}

	/**
	 * Render Author.
	 */
	protected function render_author() {
		$widget        = $this->parent;
		$settings      = $widget->get_settings();
		$light_version = $this->get_instance_value( 'light_version' );
		$author_css    = $this->get_instance_value( 'author_css' );
		?>

		 <a class = "hellix-post__meta-data <?php echo esc_attr( $author_css ); ?> <?php echo esc_attr( 'yes' === $light_version ? ' text-light uppercase' : '' ); ?>" data-barba href="#">
			<?php the_author(); ?>
		</a>
		<?php
	}

	/**
	 * Render Date.
	 */
	protected function render_date() {
		$this->render_date_by_type();
	}

	/**
	 * Render Date by Type.
	 *
	 * @param string $type type.
	 *
	 * @return void
	 */
	protected function render_date_by_type( $type = 'publish' ) {
		$widget        = $this->parent;
		$settings      = $widget->get_settings();
		$light_version = $this->get_instance_value( 'light_version' );
		$date_css      = $this->get_instance_value( 'date_css' );
		?>
		<a class ="hellix-post__meta-data <?php echo esc_attr( $date_css ); ?> <?php echo esc_attr( 'yes' === $light_version ? ' text-light' : '' ); ?>" data-barba href="#">
			<?php
			switch ( $type ) :
				case 'modified':
					$date = get_the_modified_date();
					break;
				default:
					$date = get_the_date();
			endswitch;
			/** This filter is documented in wp-includes/general-template.php */
			echo apply_filters( 'the_date', $date, get_option( 'date_format' ), '', '' ); //phpcs:ignore
			?>
		</a>
		<?php
	}

	/**
	 * Render Portfolio Filters.
	 */
	protected function portfolio_filters() {
		$portfolio_cats = array();
		$query          = $this->parent->get_query();
		while ( $query->have_posts() ) :
			$query->the_post();
			// $portfolio_types = get_the_term_list( get_the_ID(), 'jetpack-portfolio-tag', '', ', ', '' );.
			$portfolio_types = get_the_terms( get_the_ID(), 'jetpack-portfolio-tag' );

			if ( ! $portfolio_types || is_wp_error( $portfolio_types ) ) {
				$portfolio_types = array();
			}

			$portfolio_types = array_values( $portfolio_types );

			foreach ( array_keys( $portfolio_types ) as $key ) {
				_make_cat_compat( $portfolio_types[ $key ] );
			}

			foreach ( $portfolio_types as $portfolio_type ) {
				$portfolio_cats[ $portfolio_type->slug ] = $portfolio_type->name;
			}

		endwhile;
		?>
		<div class="row justify-content-center">
			<div class="col-auto">
			<div class="filter-button-group text-dark fw-500 mt-32">
				<button class="button mr-20 btn-active" data-filter="*">All</button> 

			<?php foreach ( $portfolio_cats as $key => $portfolio_cat ) : ?>

				<button class="button mr-20" data-filter=".<?php echo esc_attr( $key ); ?>" >
						<?php echo esc_html( $portfolio_cat ); ?>
				</button>
			<?php endforeach; ?>
		</div>
	</div>
</div>
		<?php
	}

	/**
	 * Render Portfolio filters Section.
	 *
	 * @param string $col_size column size.
	 * @param string $align alignment.
	 * @param string $txt_color text color.
	 * @param string $wrap_css wrapper css class.
	 */
	protected function portfolio_filters_section( $col_size = 'col-auto', $align = 'left', $txt_color = 'text-white', $wrap_css = '' ) {
		$settings       = $this->parent->get_settings();
		$portfolio_cats = array();
		$query          = $this->parent->get_query();
		while ( $query->have_posts() ) :
			$query->the_post();
			// $portfolio_types = get_the_term_list( get_the_ID(), 'jetpack-portfolio-tag', '', ', ', '' );
			$portfolio_types = get_the_terms( get_the_ID(), 'jetpack-portfolio-tag' );

			if ( ! $portfolio_types || is_wp_error( $portfolio_types ) ) {
				$portfolio_types = array();
			}

			$portfolio_types = array_values( $portfolio_types );

			foreach ( array_keys( $portfolio_types ) as $key ) {
				_make_cat_compat( $portfolio_types[ $key ] );
			}

			foreach ( $portfolio_types as $portfolio_type ) {
				$portfolio_cats[ $portfolio_type->slug ] = $portfolio_type->name;
			}

		endwhile;
		?>
		<div class="row justify-content-between <?php echo esc_attr( $wrap_css ); ?>">
			<?php if ( 'yes' === $settings['section_heading'] ) { ?>
				<div class="<?php echo esc_attr( $col_size ); ?>">
					<div class="sectionHeading -<?php echo esc_attr( $align ); ?>-line">
						<span class="sectionHeading__subtitle">
							<?php echo esc_html( $settings['pre_title'] ); ?>
						</span>
						<h2 class="sectionHeading__title <?php echo esc_attr( $txt_color ); ?>">
							<?php echo esc_html( $settings['title'] ); ?>
						</h2>
					</div>
				</div>
			<?php } ?>
			<div class="col-auto">
			<div class="filter-button-group fw-500 mt-32">
				<button class="button <?php echo esc_attr( $txt_color ); ?> mr-20 btn-active" data-filter="*">All</button> 

			<?php foreach ( $portfolio_cats as $key => $portfolio_cat ) : ?>

				<button class="button <?php echo esc_attr( $txt_color ); ?> mr-20" data-filter=".<?php echo esc_attr( $key ); ?>" >
						<?php echo esc_html( $portfolio_cat ); ?>
				</button>
			<?php endforeach; ?>
		</div>
	</div>
</div>
		<?php
	}
	/**
	 * Render Posts.
	 */
	protected function render_post() {
		$this->render_post_header();
		$this->render_thumbnail();
		$this->render_text_header();
		$this->render_title();
		$this->render_meta_data();
		$this->render_excerpt();
		$this->render_read_more();
		$this->render_text_footer();
		$this->render_post_footer();
	}
	/**
	 * Render amp.
	 */
	public function render_amp() {

	}

	/**
	 * Render script in the editor.
	 */
	public function render_script() {

	}
}
