<?php

namespace HellixElementor\Modules\Section;

use HellixElementor\Base\Module_Base;
use Elementor\Controls_Manager;
use Elementor\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * The module class.
 */
class Module extends Module_Base {

	/**
	 * Instantiate the object.
	 */
	public function __construct() {
		parent::__construct();

		$this->add_actions();
	}

	/**
	 * Get the name of the module.
	 *
	 * @return string
	 */
	public function get_name() {
		return 'override-section';
	}

	/**
	 * Actions in this module.
	 */
	public function add_actions() {
		add_action( 'elementor/frontend/section/after_render', [ $this, 'wrap_end' ], 20 );
		add_action( 'elementor/frontend/section/before_render', [ $this, 'before_render' ], 5 );
		add_action( 'elementor/element/section/section_advanced/before_section_end', [ $this, 'add_section_controls' ], 10, 2 );
		add_filter( 'elementor/section/print_template', [ $this, 'print_template' ] );
		add_action( 'elementor/element/section/section_advanced/after_section_end', [ $this, 'add_parallax_controls' ], 10, 2 );
	}

	/**
	 * Add Section Controls.
	 *
	 * @param array $element The widget.
	 * @return void
	 */
	public function add_section_controls( $element ) {
		$element->add_control(
			'container_class',
			[
				'label'       => esc_html__( 'Container CSS Classes', 'hellix-elementor' ),
				'label_block' => true,
				'type'        => Controls_Manager::TEXT,
				'description' => esc_html__( 'Applied to elementor-container element. You can use additional bootstrap utility classes here.', 'hellix-elementor' ),
			]
		);
	}

	/**
	 * Add Parallax Controls.
	 *
	 * @param array $element The widget.
	 * @param array $args The widget.
	 * @return void
	 */
	public function add_parallax_controls( $element, $args ) {

		$element->start_controls_section(
			'_section_parallax',
			[
				'label' => esc_html__( 'Parallax', 'hellix-elementor' ),
				'tab'   => Controls_Manager::TAB_ADVANCED,
			]
		);

		$element->add_control(
			'enable_parallax',
			[
				'label'   => esc_html__( 'Enable parallax', 'hellix-elementor' ),
				'type'    => Controls_Manager::SWITCHER,
				'default' => 'no',
			]
		);

		$element->add_control(
			'parallax_bg',
			[
				'label'     => esc_html__( 'Background', 'hellix-elementor' ),
				'type'      => Controls_Manager::MEDIA,
				'condition' => [
					'enable_parallax' => 'yes',
				],
			]
		);

		$element->add_control(
			'parallax_speed',
			[
				'label'     => esc_html__( 'Speed', 'hellix-elementor' ),
				'type'      => Controls_Manager::NUMBER,
				'default'   => '',
				'min'       => 0,
				'step'      => 0.1,
				'condition' => [
					'enable_parallax' => 'yes',
				],
			]
		);

		$element->add_control(
			'parallax_css',
			[
				'label'       => esc_html__( 'Parallax CSS Classes', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => '',
				'dynamic'     => [
					'active' => true,
				],
				'title'       => esc_html__( 'Add your custom class WITHOUT the dot. e.g: my-class', 'hellix-elementor' ),
				'description' => esc_html__( 'Applied to .parallax element', 'hellix-elementor' ),
				'condition'   => [
					'enable_parallax' => 'yes',
				],
			]
		);

		$element->add_control(
			'parallax_wrapper_css',
			[
				'label'       => esc_html__( 'Parallax CSS wrapper Classes', 'hellix-elementor' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => '',
				'dynamic'     => [
					'active' => true,
				],
				'title'       => esc_html__( 'Add your custom class WITHOUT the dot. e.g: my-class', 'hellix-elementor' ),
				'description' => esc_html__( 'Applied to the wrapper of .parallax element', 'hellix-elementor' ),
				'condition'   => [
					'enable_parallax' => 'yes',
				],
			]
		);

		$element->end_controls_section();
	}

	/**
	 * Wrap Start.
	 *
	 * @param array $element The widget.
	 *
	 * @return void
	 */
	public function wrap_start( $element ) {
		$settings     = $element->get_settings_for_display();
		$has_parallax = 'yes' === $settings['enable_parallax'] ? true : false;

		if ( $has_parallax ) :

			echo '<div class="custom-wrap section-bg relative">';

			$element->add_render_attribute( 'parallax_wrapper_div', 'class', 'section-bg__image' );

			$element->add_render_attribute( 'parallax_div', 'data-parallax-target' );
			$element->add_render_attribute( 'parallax_div', 'class', 'bg-image js-lazy' );

			if ( ! empty( $settings['parallax_bg']['url'] ) ) {
				$element->add_render_attribute( 'parallax_div', 'data-bg', $settings['parallax_bg']['url'] );
			}

			if ( ! empty( $settings['parallax_css'] ) ) {
				$element->add_render_attribute( 'parallax_div', 'class', $settings['parallax_css'] );
			}

			if ( ! empty( $settings['parallax_wrapper_css'] ) ) {
				$element->add_render_attribute( 'parallax_wrapper_div', 'class', $settings['parallax_wrapper_css'] );
			}

			if ( ! empty( $settings['parallax_speed'] ) ) {
				$element->add_render_attribute( 'parallax_wrapper_div', 'data-parallax', $settings['parallax_speed'] );
			}
			?>
			<div <?php $element->print_render_attribute_string( 'parallax_wrapper_div' ); ?>>
				<?php if ( isset( $settings['parallax_bg']['url'] ) && ! empty( $settings['parallax_bg']['url'] ) ) : ?>
				<div <?php $element->print_render_attribute_string( 'parallax_div' ); ?>></div>
				<?php endif; ?>
			</div>
			<?php
		endif;
	}

	/**
	 * Wrap End.
	 *
	 * @param array $element The widget.
	 *
	 * @return void
	 */
	public function wrap_end( $element ) {
		$settings     = $element->get_settings_for_display();
		$has_parallax = 'yes' === $settings['enable_parallax'] ? true : false;

		if ( $has_parallax ) :
			?>
		</div><!-- /.custom-wrap -->
			<?php
		endif;
	}

	/**
	 * Before Render.
	 *
	 * @param array $element The widget.
	 *
	 * @return void
	 */
	public function before_render( $element ) {

		$settings        = $element->get_settings();
		$container_class = $settings['gap'];

		if ( 'no' === $settings['gap'] ) {
			$container_class = $settings['gap'] . ' no-gutters';
		}

		if ( isset( $settings['container_class'] ) && ! empty( $settings['container_class'] ) ) {
			$container_class .= ' ' . $settings['container_class'];
		}

		if ( ! empty( $container_class ) ) {
			$element->set_settings( 'gap', $container_class );
		}

		$this->wrap_start( $element );

	}

	/**
	 * Print Template.
	 *
	 * @return string
	 */
	public function print_template() {
		ob_start();
			$this->content_template();
		return ob_get_clean();
	}

	/**
	 * Content Template.
	 *
	 * @return void
	 */
	public function content_template() {
		?>
		<#
		if ( settings.background_video_link ) {
			let videoAttributes = 'autoplay muted playsinline';

			if ( ! settings.background_play_once ) {
				videoAttributes += ' loop';
			}

			view.addRenderAttribute( 'background-video-container', 'class', 'elementor-background-video-container' );

			if ( ! settings.background_play_on_mobile ) {
				view.addRenderAttribute( 'background-video-container', 'class', 'elementor-hidden-phone' );
			}
		#>
			<div {{{ view.getRenderAttributeString( 'background-video-container' ) }}}>
				<div class="elementor-background-video-embed"></div>
				<video class="elementor-background-video-hosted elementor-html5-video" {{ videoAttributes }}></video>
			</div>
		<# }
			if ( 'no' == settings.gap ) {
				settings.gap = `${ settings.gap } no-gutters`;
			}

			if ( '' != settings.container_class ) {
				settings.gap = `${ settings.gap } ${ settings.container_class }`;
			}
		#>
		<div class="elementor-background-overlay"></div>
		<div class="elementor-shape elementor-shape-top"></div>
		<div class="elementor-shape elementor-shape-bottom"></div>
		<div class="elementor-container elementor-column-gap-{{ settings.gap }}">
			<?php if ( ! Plugin::$instance->experiments->is_feature_active( 'e_dom_optimization' ) ) { ?>
				<div class="elementor-row"></div>
			<?php } ?>
		</div>
		<?php
	}
}
